﻿/*
aardio-chrome 专用 websocket/json-rpc/aasdl 客户端
关于 aasdl : http://bbs.aardio.com/doc/aasdl/
*/

(function() {
  window.aardio = {};

  var urlQuery = function(variable) {
    var query = window.location.search.slice(1);
    var vars = query.split("&");
    for (var i = 0; i < vars.length; i++) {
      var pair = vars[i].split("=");
      if (pair[0] == variable) {
        return pair[1];
      }
    }
    return false;
  };

  var rpcServerPort = urlQuery("rpcServerPort");
  if (typeof rpcServerPort == "string") {
    localStorage.setItem("rpcServerPort", rpcServerPort);
  } else {
    rpcServerPort = localStorage.getItem("rpcServerPort");
    if (!rpcServerPort) rpcServerPort = "{{{$rpcServerPort}}}";
  }

  wsRpcServerUrl = "ws://127.0.0.1:" + rpcServerPort + "/rpc/ws";
  httpRpcServerUrl = "http://127.0.0.1:" + rpcServerPort + "/rpc/http";

  let ws = new WebSocket(wsRpcServerUrl);
  let rpcNotifications = new Object();

  let scripts = document.getElementsByTagName("script");
  let scriptCode = scripts[scripts.length - 1].innerHTML;

  ws.onopen = function(e) {
  };
  ws.onclose = function(e) {
    console.log("WebSocket connection closed!");
  };
  ws.onerror = function(e) {
    console.error(e);
  };
  ws.onmessage = function(e) {
    var rep = JSON.parse(e.data);
    if (typeof rep.method == "string") {
      var notify = rpcNotifications[rep.method];
      if (notify) {
        var result = notify.apply(window.aardio, rep.params);
        if (rep.id) {
          var clientRep = JSON.stringify({
            id: rep.id,
            jsonrpc: "2.0",
            result: result
          });
          ws.send(clientRep);
        }
      }
    }
  };

  window.aardio.on = function(method, notify) {
    rpcNotifications[method] = notify;
  };

  window.aardio.on("doScript",(js)=>{
    window.eval ("(()=>{" + js + "})()"); 
  })

  window.aardio.on("rpcClientId", id => {
    window.aardio.rpcClientId = id;
    window.aardio
      .xcall("?")
      .then(function(aasdl, error) {
        var paseraasdl = function(obj, ex, pk) {
          for (var k in obj) {
            let method = k;
            if (typeof pk == "string") method = pk + "." + k;

            if (typeof obj[k] == "object") {
              ex[k] = {};
              paseraasdl(obj[k], ex[k], method);
              continue;
            }

            ex[k] = function() {
              return window.aardio.xcall.apply(ex, [method, ...arguments]);
            };
          }
        };

        paseraasdl(JSON.parse(aasdl), window.aardio);

        eval(scriptCode);
        window.aardio.xcall("$onLoadUrl", document.location.href);
      })
      .catch(e => console.error(e));
  });

  window.aardio.xcall = function(method,...args) {

    var reqData = JSON.stringify({
      method: method,
      id: method,
      jsonrpc: "2.0",
      params: args
    });

    return new Promise((resolve, reject) => {
      fetch(httpRpcServerUrl, {
        body: reqData,
        cache: "no-cache",
        headers: {
          "Content-Type": "application/json",
          "Rpc-Client-Id": window.aardio.rpcClientId + ""
        },
        method: "POST",
        mode: "cors"
      })
        .then(response => response.json())
        .then(json => {
          
          if (json.error) {
            console.error("调用aardio函数时遇到错误,请求数据：",reqData)
            reject(json.error);
          }
          else resolve(json.result);
        })
        .catch(e => reject(e));
    });
  };
})();
